/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include <fggl/ecs/ecs.hpp>

#include <iostream>

using namespace fggl::ecs;

Archetype::Archetype(const archToken_t& token ) : type(token) {
	for ( archToken_t::size_type i = 0; i < token.size(); ++i ) {
		data.push_back( new unsigned char[default_cap] );
		dataSizes.push_back( default_cap );
	}
}

ECS::ECS() : m_entityIDCounter(1) {}

ECS::~ECS() {
	for( Archetype* arch : m_archetypes ) {
		for ( std::size_t i=0; i < arch->type.size(); ++i ) {
			const ComponentBase* const comp = m_componentMap[arch->type[i]];
			const std::size_t& size = comp->size();

			for ( std::size_t e=0; e<arch->entities.size(); ++e ) {
				comp->destroy(&arch->data[i][e*size]);
			}
			delete[] arch->data[i];
		}
		delete arch;
	}

	for( componentmap_t::value_type& p : m_componentMap )
		delete p.second;
}

entity_t ECS::getNewID() {
	return m_entityIDCounter++;
}

entity_t ECS::createEntity( ) {
	auto eid = getNewID();

	Record dummy;
	dummy.archetype = nullptr;
	dummy.index = 0;
	m_entityArchtypes[ eid ] = dummy;

	return eid;
}

Archetype* ECS::getArchetype(const archToken_t& id) {
	for ( auto* arch : m_archetypes ) {
		if ( arch->type == id ) {
			return arch;
		}
	}

	// didn't exist, need to make one
	Archetype* arch = new Archetype(id);
	m_archetypes.push_back( arch );

	return arch;
}
