/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_AUDIO_AUDIO_HPP
#define FGGL_AUDIO_AUDIO_HPP

#include <string>

namespace fggl::audio {

	/**
	 * AudioClip is bit of audio loaded into memory.
	 *
	 * If the sampleCount is -1, the clip is invalid.
	 */
	struct AudioClip {
		int channels;
		int sampleRate;
		int sampleCount;
		short* data;
	};

	class AudioService {
		public:
			AudioService() = default;
			virtual ~AudioService() = default;

			virtual void play(const std::string& filename, bool looping = false) = 0;
			virtual void play(AudioClip& clip, bool looping = false) = 0;
	};

} // namespace fggl::audio

#endif //FGGL_AUDIO_AUDIO_HPP
