/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_DATA_HEIGHTMAP_HPP
#define FGGL_DATA_HEIGHTMAP_HPP

#include <cstdint>
#include <array>

#include "fggl/data/model.hpp"

namespace fggl::data {

	constexpr std::size_t heightMaxX = 255;
	constexpr std::size_t heightMaxZ = 255;
	constexpr float heightSeaLevel = 0.0f;

	struct HeightMap {
		constexpr static const char name[] = "Heightmap";
		std::array<float, heightMaxX * heightMaxZ> heightValues;

		void clear() {
			for (float &heightValue : heightValues) {
				heightValue = heightSeaLevel;
			}
		}

		[[nodiscard]]
		inline float getValue(std::size_t x, std::size_t z) const {
			return heightValues.at(x * heightMaxZ + z);
		}

		inline void setValue(std::size_t x, std::size_t z, float value) {
			heightValues.at(x * heightMaxZ + z) = value;
		}
	};

	inline int idx(int x, int z, int zMax) {
		return x * zMax + z;
	}

	void generateHeightMesh(const data::HeightMap *heights, data::Mesh &mesh);
}

#endif //FGGL_DATA_HEIGHTMAP_HPP
