/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GFX_COMPAT_HPP
#define FGGL_GFX_COMPAT_HPP
/**
 * Window management Calls.
 *
 * This shouldn't be exposed to the demo app, but the ECS we're using isn't smart enouph to allow us to
 * abstract this yet. It's next thing on the list, but this branch is about cleaning up OpenGL not about
 * extending our ECS.
 *
 * Should be removed when the engine has suitable abstractions in place.
 */

#include <memory>

#include <utility>
#include <fggl/gfx/window.hpp>
#include <fggl/ecs3/ecs.hpp>

namespace fggl::gfx {

	//
	// fake module support - allows us to still RAII
	//
	struct ecsGlfwModule : ecs3::Module {

		inline explicit
		ecsGlfwModule(std::shared_ptr<fggl::input::Input> inputs) : context( std::make_shared<GlfwContext>(std::move(inputs))) {
		}

		inline
		std::unique_ptr<GlfwWindow> createWindow(const std::string &title) {
			std::unique_ptr<GlfwWindow> window = std::make_unique<GlfwWindow>(context);
			window->title(title);
			return std::move(window);
		}

		void onUpdate() override {
			context->pollEvents();
		}

		[[nodiscard]]
		std::string name() const override {
			return "gfx::glfw";
		}

		private:
			std::shared_ptr<GlfwContext> context;

	};

	using Glfw = ecsGlfwModule;
}

#endif
