/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GFX_OGL_COMPAT_HPP
#define FGGL_GFX_OGL_COMPAT_HPP
/**
 * Legacy/Direct OpenGL calls.
 *
 * This shouldn't be exposed to the demo app, but the ECS we're using isn't smart enouph to allow us to
 * abstract this yet. It's next thing on the list, but this branch is about cleaning up OpenGL not about
 * extending our ECS.
 *
 * Should be removed when the engine has suitable abstractions in place.
 */

#include <functional>

#include <fggl/gfx/ogl/shader.hpp>
#include <fggl/gfx/ogl/renderer.hpp>

#include <fggl/gfx/common.hpp>
#include <fggl/gfx/camera.hpp>
#include <fggl/ecs/ecs.hpp>
#include <utility>
#include <fggl/input/camera_input.hpp>
#include <fggl/data/heightmap.hpp>

#include "fggl/gfx/phong.hpp"

namespace fggl::gfx {

	//
	// fake module support - allows us to still RAII
	//
	struct SceneUtils : ecs3::Module {

		SceneUtils() = default;

		[[nodiscard]]
		std::string name() const override {
			return "gfx::scene";
		}

		void onLoad(ecs3::ModuleManager &manager, ecs3::TypeRegistry &types) override {
			// mesh dependencies
			types.make<math::Transform>();
			types.make<data::StaticMesh>();
			types.make<data::HeightMap>();

			types.make<gfx::PhongMaterial>();

			// camera dependencies
			types.make<fggl::gfx::Camera>();
			types.make<fggl::input::FreeCamKeys>();
		}

	};

}

#endif
