/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 22/06/22.
//

#ifndef FGGL_GFX_OGL4_FALLBACK_HPP
#define FGGL_GFX_OGL4_FALLBACK_HPP

/**
 * Fallback shaders.
 *
 * Embedded in the library, so that the user can use the library without having to include the shaders.
 */
namespace fggl::gfx::ogl4 {

	constexpr const char* FALLBACK_CANVAS_PIPELINE = "fallback_canvas";

	constexpr const char* FALLBACK_CANVAS_VERTEX_SHADER = R"glsl(
		#version 330 core
		layout (location = 0) in vec2 aPos;
		layout (location = 1) in vec3 aColour;
		layout (location = 2) in vec2 aTexPos;

		uniform mat4 projection;

		out vec3 colour;
		out vec2 texPos;

		void main() {
			gl_Position = projection * vec4(aPos, 0.0, 1.0);
			colour = aColour;
			texPos = aTexPos;
		})glsl";

	constexpr const char* FALLBACK_CANVAS_FRAGMENT_SHADER = R"glsl(
		#version 330 core
		uniform sampler2D tex;

		in vec3 colour;
		in vec2 texPos;

		out vec4 fragColour;

		void main() {
			fragColour = vec4(colour.xyz, texture(tex, texPos).r);
		})glsl";

} // namespace fggl::gfx::ogl4

#endif //FGGL_GFX_OGL4_FALLBACK_HPP
