/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 05/06/22.
//

#ifndef FGGL_GFX_PHONG_HPP
#define FGGL_GFX_PHONG_HPP

#include "fggl/math/types.hpp"

namespace fggl::gfx {

	struct PhongMaterial {
		constexpr static const char* name = "gfx::material";
		math::vec3 ambient;
		math::vec3 diffuse;
		math::vec3 specular;
		float shininess;
	};

	constexpr math::vec3 DEFAULT_AMBIENT { 0.05F, 0.05F, 0.05F};
	constexpr math::vec3 DEFAULT_DIFFUSE { 0.5F, 0.5F, 0.5F};
	constexpr math::vec3 DEFAULT_SPECULAR { 0.7F, 0.7F, 0.7F};
	constexpr float DEFAULT_SHININESS = .078125F;

	constexpr PhongMaterial DEFAULT_MATERIAL {
		DEFAULT_AMBIENT,
		DEFAULT_DIFFUSE,
		DEFAULT_SPECULAR,
		DEFAULT_SHININESS
	};


	enum class LightType {
			Directional,
			Point,
			Spot
	};

	struct Light {
		constexpr static const char* name = "gfx::light";
		LightType type;
		math::vec3 position;

		// colours
		math::vec3 ambient;
		math::vec3 diffuse;
		math::vec3 specular;

		// distance data
		math::vec3 falloffs; // (constant, linear, quadratic)
		float cutOff;
		float outerCutOff;
	};

} // namesapce fggl::gfx

#endif //FGGL_GFX_PHONG_HPP
