/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GFX_WINDOWING_HPP
#define FGGL_GFX_WINDOWING_HPP

#include <memory>
#include <fggl/gfx/paint.hpp>
#include "fggl/ecs3/ecs.hpp"

namespace fggl::gfx {

	struct Bounds {
		float top;
		float right;
		float bottom;
		float left;
	};

	class Graphics {
		public:
			virtual ~Graphics() = default;
			virtual void clear() = 0;
			virtual void resize(int width, int height) = 0;

			virtual Bounds canvasBounds() = 0;
			virtual void draw2D(const Paint &paint) = 0;

			virtual void drawScene(ecs3::World&) = 0;
	};

	class Window {
		public:
			virtual ~Window() = default;
			virtual void activate() const = 0;

			template<typename T>
			void make_graphics() {
				activate();
				m_graphics = std::make_unique<T>(*this);
			}

			// window-related getters
			[[nodiscard]]
			virtual math::vec2i frameSize() const = 0;

			[[nodiscard]]
			virtual bool wantClose() const = 0;

			// window callbacks
			virtual void frameStart() = 0;
			virtual void frameEnd() = 0;

			Graphics &graphics() {
				return *m_graphics;
			}

		protected:
			std::unique_ptr<Graphics> m_graphics;
	};

	class WindowService {
		public:
			Window &create();
	};

} // namespace fggl::gfx

#endif
