/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_INPUT_CAMERA_INPUT_HPP
#define FGGL_INPUT_CAMERA_INPUT_HPP

#include <fggl/ecs3/ecs.hpp>
#include <fggl/math/types.hpp>

namespace fggl::input {

	constexpr float ROT_SPEED = 0.05f;
	constexpr float PAN_SPEED = 0.05f;
	constexpr math::mat4 MAT_IDENTITY(1.0f);

	struct FreeCamKeys {
		constexpr const static char name[] = "FreeCameraKeys";
		scancode_t forward;
		scancode_t backward;
		scancode_t left;
		scancode_t right;
		scancode_t rotate_cw;
		scancode_t rotate_ccw;
	};

	/**
	 * Process the camera based on rotation around a fixed point.
	 *
	 * @param ecs the world that contains the camera
	 * @param input the input module to read the mouse location from
	 * @param cam the ID of the camera entity
	 */
	void process_arcball(fggl::ecs3::World &ecs, const Input &input, fggl::ecs::entity_t cam);

	/**
	 * Process free (floating) camera movement.
	 *
	 * @param ecs the world that contains the camera
	 * @param input the input module to read the mouse location from
	 * @param cam the ID of the camera entity
	 */
	void process_freecam(fggl::ecs3::World &ecs, const Input &input, fggl::ecs::entity_t cam);

	/**
	 * Input processing for moving the camera when the mouse is close to the edge of the screen.
	 *
	 * This function deals with ensuring the camera moves if the user moves their mouse button close
	 * to the edge of the screen. It will apply this as movement to the camera entity passed in as
	 * an argument.
	 *
	 * @param ecs the world that contains the camera
	 * @param input the input module to read the mouse location from
	 * @param cam the ID of the camera entity
	 */
	void process_edgescroll(fggl::ecs3::World &ecs, const Input &input, fggl::ecs::entity_t cam);

}

#endif //FGGL_INPUT_CAMERA_INPUT_HPP
