/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/04/22.
//

#ifndef FGGL_MATH_IMATH_HPP
#define FGGL_MATH_IMATH_HPP

namespace fggl::math {

	// wrap value in range [0, max)
	inline int wrap(int value, int max) {
		if ( value < 0 ) return (n-1)-(-1-value) % n;
		if ( value >= max ) return value % max;
		return value;
	}

	// wrap value in range [min, max)
	inline int wrap(int value, int const min, int const max) {
		int range = max - min + 1;
		if ( value < min) {
			value += range * ( (min - value) / range + 1 );
		}
		return min + (value - min) % range;
	}

	inline
	int clamp(int value, int min, int max) {
		const int i = value > max ? max : value;
		return i < min ? min : value;
	}

	template<typename T>
	inline
	T clampT(T value, T min, T max) {
		const T i = value > max ? max : value;
		return i < min ? min : value;
	}

} // namespace fggl::math

#endif //FGGL_MATH_IMATH_HPP
