/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 24/04/22.
//

#ifndef FGGL_PHYS_BULLET_BULLET_HPP
#define FGGL_PHYS_BULLET_BULLET_HPP

#include "fggl/ecs3/module/module.hpp"

#include "fggl/phys/types.hpp"
#include "fggl/phys/bullet/types.hpp"

namespace fggl::phys::bullet {

	/**
	 * Bullet integration module.
	 *
	 * This provides the ability for FGGL to use bullet as a physics backened. Bullet is mature, stable physics
	 * engine which makes it suitable for most use cases. For use with FGGL there is a reasonable amount of copying
	 * back and forth to keep the two states in sync.
	 */
	struct BulletModule : ecs3::Module {
		public:
			BulletModule() = default;

			[[nodiscard]]
			std::string name() const override {
				return "phys::Bullet3";
			}

			void onLoad(ecs3::ModuleManager & /*manager*/, ecs3::TypeRegistry &types) override {
				// dependencies
				types.make<phys::CollisionCallbacks>();
				types.make<phys::CollisionCache>();

				types.make<phys::RigidBody>();
				types.make<phys::Dynamics>();

				// my types
				types.make<phys::bullet::BulletBody>();
			}

	};

} // namespace fggl::phys::bullet

namespace fggl::phys {

	// allows using fggl::phys::Bullet3 as the module name
	using Bullet3 = bullet::BulletModule;

} // namespace fggl::phys

#endif //FGGL_PHYS_BULLET_BULLET_HPP
