/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 11/06/22.
//

#ifndef FGGL_PHYS_BULLET_MOTION_HPP
#define FGGL_PHYS_BULLET_MOTION_HPP

#include "fggl/phys/bullet/types.hpp"

namespace fggl::phys::bullet {

	class FgglMotionState : public btMotionState {
		public:
			FgglMotionState(fggl::ecs3::World* world, fggl::ecs3::entity_t entity) : m_world(world), m_entity(entity) {
			}
			virtual ~FgglMotionState() = default;

			void getWorldTransform(btTransform& worldTrans) const override {
				const auto* transform = m_world->get<fggl::math::Transform>(m_entity);
				worldTrans.setFromOpenGLMatrix( glm::value_ptr(transform->model()) );
			}

			void setWorldTransform(const btTransform& worldTrans) override {
				auto* transform = m_world->get<fggl::math::Transform>(m_entity);

				// set position
				auto btOrigin = worldTrans.getOrigin();
				transform->origin( {btOrigin.x(), btOrigin.y(), btOrigin.z()} );

				// set rotation
				math::vec3 angles;
				worldTrans.getRotation().getEulerZYX(angles.x, angles.y, angles.z);
				transform->euler(angles);
			}

		private:
			fggl::ecs3::World* m_world;
			fggl::ecs3::entity_t m_entity;
	};

} // namespace fggl::phys::bullet

#endif //FGGL_PHYS_BULLET_INTEGRATIONS_HPP
