/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 23/04/22.
//

#ifndef FGGL_SCENES_GAME_HPP
#define FGGL_SCENES_GAME_HPP

#include "fggl/app.hpp"
#include "fggl/phys/types.hpp"

namespace fggl::scenes {

	class Game : public fggl::AppState {

		public:
			explicit Game(fggl::App& app);

			void activate() override;
			void deactivate() override;

			void update() override;
			void render(fggl::gfx::Graphics& gfx) override;

		protected:
			inline auto world() -> ecs3::World& {
				return *m_world;
			}

			inline auto input() -> input::Input& {
				return *m_input;
			}

		private:
			std::shared_ptr<input::Input> m_input;
			std::unique_ptr<ecs3::World> m_world;
			std::unique_ptr<phys::PhysicsEngine> m_phys;
			std::string m_previous = "menu";
	};

} // namespace fggl::scenes

#endif //FGGL_SCENES_GAME_HPP
