/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_UTIL_SERVICE_HPP
#define FGGL_UTIL_SERVICE_HPP

#include <memory>
#include <typeindex>
#include <unordered_map>
#include <stdexcept>
#include <iostream>

namespace fggl::util {

	class ServiceLocator {
			inline static ServiceLocator *s_instance;
			std::unordered_map<std::type_index, std::shared_ptr<void>> m_services;
			ServiceLocator() = default;

		public:
			// don't allow copying or moving
			ServiceLocator(ServiceLocator &other) = delete;
			ServiceLocator(ServiceLocator &&other) = delete;

			static ServiceLocator &instance() {
				if (s_instance == nullptr) {
					s_instance = new ServiceLocator();
				}
				return *s_instance;
			}

			template<typename T>
			std::shared_ptr<T> supply(std::shared_ptr<T> ptr) {
				auto info = std::type_index(typeid(T));
				m_services[info] = ptr;
				return ptr;
			}

			template<typename T>
			std::shared_ptr<T> get() {
				try {
					auto info = std::type_index(typeid(T));
					return std::static_pointer_cast<T>(m_services.at(info));
				} catch ( std::out_of_range& e ){
					std::cerr << "someone requested a service that doesn't exist!" << std::endl;
					return nullptr;
				}
			}

			template<typename T>
			std::shared_ptr<T> providePtr() {
				return get<T>();
			}

	};

}

#endif //FGGL_UTIL_SERVICE_HPP
